#region ... Copyright  2003 Roland Weigelt ...
/*
Copyright  2003 Roland Weigelt.

This software is supplied "AS IS".  The author disclaims all
warranties, expressed or implied, including, without limitation,
the warranties of merchantability and of fitness for any purpose.
The author assumes no liability for direct, indirect, incidental,
special, exemplary, or consequential damages, which may result
from the use of this software, even if advised of the possibility
of such damage.

Permission is hereby granted to use, copy, modify, and distribute
this source code, or portions hereof, for any purpose, without
fee, subject to the following restrictions:

1. The origin of this source code must not be misrepresented.

2. Altered versions must be plainly marked as such and must not
be misrepresented as being the original source.

3. This Copyright notice may not be removed or altered from any
source or altered source distribution.

The author specifically permits, without fee, the use of this
source code as a component in commercial products.  If you use
this source code in a product, acknowledgment is not required but
would be appreciated.
*/
#endregion

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace Weigelt.Windows.Forms
{
	/// <summary>
	/// A very simple control for writing output like on an old terminal.
	/// </summary>
	public class OldTerminal : System.Windows.Forms.UserControl
	{
		//===================================================
		// Private
		//===================================================

		#region ... Private data ...

		private const string _EmptyRtf=@"{\rtf1\ansi }";

		private int m_nCharactersPerSecond=40;
		private string m_strBuffer=String.Empty;
		private int m_nBufferIndex=0;
		private bool m_bWritingOutput=false;

		private System.Windows.Forms.RichTextBox c_rtfOutput;
		private System.Windows.Forms.Timer c_tmrOutput;
		private System.ComponentModel.IContainer components;

		#endregion

		#region ... Component Designer generated code ...
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.c_rtfOutput = new System.Windows.Forms.RichTextBox();
			this.c_tmrOutput = new System.Windows.Forms.Timer(this.components);
			this.SuspendLayout();
			// 
			// c_rtfOutput
			// 
			this.c_rtfOutput.BackColor = System.Drawing.Color.Black;
			this.c_rtfOutput.DetectUrls = false;
			this.c_rtfOutput.Dock = System.Windows.Forms.DockStyle.Fill;
			this.c_rtfOutput.Font = new System.Drawing.Font("Lucida Console", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.c_rtfOutput.ForeColor = System.Drawing.Color.FromArgb(((System.Byte)(0)), ((System.Byte)(192)), ((System.Byte)(0)));
			this.c_rtfOutput.Location = new System.Drawing.Point(0, 0);
			this.c_rtfOutput.Name = "c_rtfOutput";
			this.c_rtfOutput.ReadOnly = true;
			this.c_rtfOutput.ScrollBars = System.Windows.Forms.RichTextBoxScrollBars.None;
			this.c_rtfOutput.Size = new System.Drawing.Size(264, 192);
			this.c_rtfOutput.TabIndex = 0;
			this.c_rtfOutput.Text = "";
			this.c_rtfOutput.KeyDown += new System.Windows.Forms.KeyEventHandler(this.HandleOutputKeyDown);
			this.c_rtfOutput.MouseDown += new System.Windows.Forms.MouseEventHandler(this.HandleOutputMouseDown);
			this.c_rtfOutput.SelectionChanged += new System.EventHandler(this.HandleOutputSelectionChanged);
			// 
			// c_tmrOutput
			// 
			this.c_tmrOutput.Interval = 25;
			this.c_tmrOutput.Tick += new System.EventHandler(this.HandleTimerTick);
			// 
			// OldTerminal
			// 
			this.BackColor = System.Drawing.Color.Black;
			this.Controls.Add(this.c_rtfOutput);
			this.Name = "OldTerminal";
			this.Size = new System.Drawing.Size(264, 192);
			this.ResumeLayout(false);

		}
		#endregion

		#region ... Event handlers ...

		private void HandleTimerTick(object sender, System.EventArgs e)
		{
			if (m_nBufferIndex>=m_strBuffer.Length)
				Stop();
			else
				WriteRawText(m_strBuffer.Substring(m_nBufferIndex++,1));
		}

		private void HandleOutputKeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			e.Handled=true;
		}

		private void HandleOutputSelectionChanged(object sender, System.EventArgs e)
		{
			if (! m_bWritingOutput)
				GotoEndOfText();
		}

		private void HandleOutputMouseDown(object sender, MouseEventArgs e)
		{
			if (! m_bWritingOutput)
				GotoEndOfText();
		}
		#endregion

		#region ... Protected methods ...

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}


		/// <summary>
		/// Sets the caret to the end of the text
		/// </summary>
		protected void GotoEndOfText()
		{
			c_rtfOutput.SelectionLength=0;
			c_rtfOutput.SelectionStart=c_rtfOutput.TextLength;
			c_rtfOutput.Focus();
		}

		/// <summary>
		/// Writes the specified text in one go.
		/// </summary>
		/// <param name="text"></param>
		protected void WriteRawText(string text)
		{
			m_bWritingOutput=true;
			GotoEndOfText();

			c_rtfOutput.SelectedText=text;
			c_rtfOutput.ScrollToCaret();
			m_bWritingOutput=false;
		}

		#endregion

		//===================================================
		// Public
		//===================================================

		#region ... Public properties ...

		/// <summary>
		/// Gets or sets the background color for the control.
		/// </summary>
		public override Color BackColor
		{
			get { return c_rtfOutput.BackColor; }
			set {	c_rtfOutput.BackColor=value; }
		}

		/// <summary>
		/// Gets or sets the forground color for the control.
		/// </summary>
		public override Color ForeColor
		{
			get { return c_rtfOutput.ForeColor; }
			set {	c_rtfOutput.ForeColor=value; }
		}

		/// <summary>
		/// Gets or sets the font for the control.
		/// </summary>
		public override Font Font
		{
			get { return c_rtfOutput.Font; }
			set {	c_rtfOutput.Font=value; }
		}

		/// <summary>
		/// Gets or sets the speed of the text output
		/// </summary>
		[
			Category("Behavior"),
			Description("The speed of the text output")
		]
		public int CharactersPerSecond
		{
			get { return m_nCharactersPerSecond; }
			set
			{
				if ((value<=0) || (value>=1000))
					throw new ArgumentOutOfRangeException("value",value,"Only values between 1 and 999 are allowed.");
				m_nCharactersPerSecond=value;
				c_tmrOutput.Interval=1000/m_nCharactersPerSecond;
			}
		}

		#endregion

		#region ... Public methods ...
		/// <summary>
		/// Clears the complete output
		/// </summary>
		public void ClearScreen()
		{
			if (c_tmrOutput.Enabled)
				c_tmrOutput.Enabled=false;
			m_strBuffer=String.Empty;
			m_nBufferIndex=0;
			c_rtfOutput.Rtf=_EmptyRtf;
		}

		/// <summary>
		/// Writes the specified string value to the output area.
		/// </summary>
		/// <param name="text"></param>
		public void Write(string text)
		{
			m_strBuffer+=text;
			if (! c_tmrOutput.Enabled)
				c_tmrOutput.Enabled=true;
		}

		/// <summary>
		/// Writes the current line terminator to the output area.
		/// </summary>
		public void WriteLine()
		{
			this.Write("\n");
		}

		/// <summary>
		/// Writes the specified string value, followed by the current line terminator, to the output area.
		/// </summary>
		/// <param name="text"></param>
		public void WriteLine(string text)
		{
			this.Write(text);
			this.WriteLine();
		}

		/// <summary>
		/// Stops the output immediately, throwing away any
		/// text in the buffer.
		/// </summary>
		public void Stop()
		{
			c_tmrOutput.Enabled=false;
			m_strBuffer=String.Empty;
			m_nBufferIndex=0;
		}

		#endregion

		#region ... Constructor ...

		/// <summary>
		/// Creates a new <see cref="OldTerminal"/> instance.
		/// </summary>
		public OldTerminal()
		{
			InitializeComponent(); // This call is required by the Windows.Forms Form Designer.
		}

		#endregion
	}
}
